#!/usr/bin/env python3
# -*- coding: utf-8 -*-
""" unit test for Hoya optical glass catalog

.. codeauthor: Michael J. Hayford
"""

import unittest
import opticalglass.hoya as h


class HoyaTestCase(unittest.TestCase):
    catalog = h.HoyaCatalog()

    def compare_indices(self, glass, tol=5e-6):
        nC = glass.rindex(656.27)
        nd = glass.rindex(587.56)
        ne = glass.rindex(546.07)
        nF = glass.rindex(486.13)
        ng = glass.rindex(435.84)
        nh = glass.rindex(404.66)
        nI = glass.rindex(365.01)
        indxC = glass.glass_data()[self.catalog.data_index('nC')]
        indxd = glass.glass_data()[self.catalog.data_index('nd')]
        indxe = glass.glass_data()[self.catalog.data_index('ne')]
        indxF = glass.glass_data()[self.catalog.data_index('nF')]
        indxg = glass.glass_data()[self.catalog.data_index('ng')]
        indxh = glass.glass_data()[self.catalog.data_index('nh')]
        indxI = glass.glass_data()[self.catalog.data_index('ni')]
        self.assertAlmostEqual(nC, indxC, delta=tol)
        self.assertAlmostEqual(nd, indxd, delta=tol)
        self.assertAlmostEqual(ne, indxe, delta=tol)
        self.assertAlmostEqual(nF, indxF, delta=tol)
        self.assertAlmostEqual(ng, indxg, delta=tol)
        self.assertAlmostEqual(nh, indxh, delta=tol)
        self.assertAlmostEqual(nI, indxI, delta=tol)

    def test_hoya_catalog_glass_index(self):
        fc5 = self.catalog.glass_index('FC5')  # first in list
        self.assertEqual(fc5, 0)
        fcd1 = self.catalog.glass_index('FCD1')
        self.assertEqual(fcd1, 1)
        ef2 = self.catalog.glass_index('E-F2')
        self.assertEqual(ef2, 28)
        bsc7 = self.catalog.glass_index('BSC7')
        self.assertEqual(bsc7, 11)
        mctaf1 = self.catalog.glass_index('MC-TAF1')  # last in list
        self.assertEqual(mctaf1, 193)

    def test_hoya_catalog_data_index(self):
        nd = self.catalog.data_index('nd')
        self.assertEqual(nd, 4)
        vd = self.catalog.data_index('νd')
        self.assertEqual(vd, 5)
        A0 = self.catalog.data_index('A0')
        self.assertEqual(A0, 28)
        n1529 = self.catalog.data_index('n1529.6')
        self.assertEqual(n1529, 10)

    def test_hoya_glass_fcd1(self):
        fcd1 = h.HoyaGlass('FCD1')
        self.assertIsNotNone(fcd1.gindex)
        self.assertEqual(fcd1.name(), 'FCD1')
        self.compare_indices(fcd1)

    def test_hoya_glass_ef2(self):
        ef2 = h.HoyaGlass('E-F2')
        self.assertIsNotNone(ef2.gindex)
        self.assertEqual(ef2.name(), 'E-F2')
        self.compare_indices(ef2)

    def test_hoya_glass_bsc7(self):
        bsc7 = h.HoyaGlass('BSC7')
        self.assertIsNotNone(bsc7.gindex)
        self.assertEqual(bsc7.name(), 'BSC7')
        self.compare_indices(bsc7)


if __name__ == '__main__':
    unittest.main(verbosity=2)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Copyright © 2019 Michael J. Hayford
""" Update support for commercial glass catalogs

.. codeauthor: Michael J. Hayford
"""


def find_differences(catalog_prev, catalog):
    """ find differences in the glass list between 2 catalog versions """
    gnames_prev = catalog_prev.get_glass_names()
    gnames = catalog.get_glass_names()

    num_glasses_prev = len(gnames_prev)
    num_glasses = len(gnames)

    gnames_prev_set = set(gnames_prev)
    gnames_set = set(gnames)

    gnames_intersection = gnames_prev_set.intersection(gnames_set)
    gnames_unchanged = len(gnames_intersection)

    gnames_removed = gnames_prev_set.difference(gnames_intersection)
    gnames_added = gnames_set.difference(gnames_intersection)

    print('previous number of glasses:', num_glasses_prev)
    print('current number of glasses:', num_glasses)
    print('number of unchanged glasses:', gnames_unchanged)

    num_gnames_removed = len(gnames_removed)
    print('number of glasses removed:', num_gnames_removed)
    if num_gnames_removed > 0:
        print(gnames_removed)

    num_gnames_added = len(gnames_added)
    print('number of glasses added:', num_gnames_added)
    if num_gnames_added > 0:
        print(gnames_added)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Copyright © 2019 Michael J. Hayford
""" Support for Glass catalogs and instances

The ``glass`` module contains the two base classes fundamental to the
:mod:`opticalglass` module. The :class:`~opticalglass.glass.GlassCatalog` class
implements as much of the common functionality needed for access to the
catalog data as possible.

The :class:`~opticalglass.glass.Glass` is an interface to the data for a particular
glass in a catalog. The primary function of interest for optical calculations
is :func:`~opticalglass.glass.Glass.rindex` which returns the refractive index at the
input wavelength (nm).

A factory interface to ``Glass`` creation is the function
:func:`~opticalglass.glassfactory.create_glass` that returns a ``Glass``
instance of the appropriate catalog type, given the glass and catalog names.

.. codeauthor: Michael J. Hayford
"""
import logging
from pathlib import Path

import xlrd
import numpy as np

from . import glasserror as ge


spectra = {'Nd': 1060.0,
           't': 1013.98,
           's': 852.11,
           'r': 706.5188,
           'C': 656.2725,
           "C'": 643.8469,
           'HeNe': 632.8,
           'D': 589.2938,
           'd': 587.5618,
           'e': 546.074,
           'F': 486.1327,
           "F'": 479.9914,
           'g': 435.8343,
           'h': 404.6561,
           'i': 365.014}
""" dict:
       - keys: spectral line labels
       - values: wavelengths in nm
"""


def get_filepath(fname):
    """ given a (spreadsheet) file name, return a complete Path to the file

    The data files included with the ``opticalglass`` package are located in
    a data directory in the package hierarchy.
    ::

        opticalglass/
            opticalglass/
                data/
                    fname

    Args:
        fname (str): the spreadsheet filename, including extender

    Returns:
        str: full path including filename for requested spreadsheet
    """
    pth = Path(__file__).resolve().parent
    return pth/'data'/fname


class GlassCatalog:
    """ base glass catalog

    Args:
        name: name of the glass catalog
        fname: excel filename, located in ``data`` directory
        glass_str: the header string for the Glass column in fname
        coef_str: the header string for the first refractive index coefficient
                  column in fname
        rindex_str: the header string for the first refractive index value
                    column in fname

    Attributes:
        num_glasses: number of glasses in the catalog
        data_header: the row containing the data header labels
        data_start: first row in the spreadsheet contain glass data
        name_col_offset: the column offset for glass_str
        coef_col_offset: the column offset for coef_str
        index_col_offset: the column offset for rindex_str
        data_header_offset: the row offset of the data headers from the
                            glass_str row
        nline_str: a dict of strings mapping header strings used for index for
                   spectral lines. **Must be provided by the derived class**
    """

    def __init__(self, name, fname, glass_str, coef_str, rindex_str,
                 data_header_offset=0):
        self.name = name
        # Open the workbook
        xl_workbook = xlrd.open_workbook(get_filepath(fname))
        self.xl_data = xl_workbook.sheet_by_index(0)

        for i in range(0, self.xl_data.nrows):
            try:
                self.name_col_offset = (self.xl_data.row_values(i, 0)
                                        .index(glass_str))
                glass_header = i
                self.data_header = i
                # the data headers may be offset from the Glass header row
                self.data_header += data_header_offset
                break
            except ValueError:
                pass

        for j in range(glass_header+1, self.xl_data.nrows):
            gname = self.xl_data.cell_value(j, self.name_col_offset)
            if len(gname) > 0:
                self.data_start = j
                break

        gnames = self.get_glass_names()
        self.num_glasses = len(gnames)

        colnames = self.xl_data.row_values(self.data_header, 0)
        self.coef_col_offset = colnames.index(coef_str)
        self.index_col_offset = colnames.index(rindex_str)

    def get_glass_names(self):
        """ returns a list of glass names """
        gnames = self.xl_data.col_values(self.name_col_offset, self.data_start)
        # filter out any empty cells at the end
        while gnames and len(gnames[-1]) is 0:
            gnames.pop()
        return gnames

    def glass_index(self, gname):
        """ returns the glass index (row) for glass name `gname`

        Args:
            gname (str): glass name

        Returns:
            int: the index (row) of the requested glass

        Raises:
            GlassNotFoundError: if **gname** doesn't match any header string
        """
        gnames = self.xl_data.col_values(self.name_col_offset, self.data_start)
        if gname in gnames:
            gindex = gnames.index(gname)
        else:
            logging.info('%s glass %s not found', self.name, gname)
            raise ge.GlassNotFoundError(self.name, gname)

        return gindex

    def data_index(self, dname):
        """ returns the data index (column) for data `dname`

        Args:
            dname (str): header string for data

        Returns:
            int: the index (column) of the requested data

        Raises:
            GlassDataNotFoundError: if **dname** doesn't match any header string
        """
        if dname in self.xl_data.row_values(self.data_header, 0):
            dindex = self.xl_data.row_values(self.data_header, 0).index(dname)
        else:
            logging.info('%s glass data type %s not found', self.name, dname)
            raise ge.GlassDataNotFoundError(self.name, dname)

        return dindex

    def glass_data(self, gindex):
        """ returns an array of data for the glass at **gindex** """
        return self.xl_data.row_values(self.data_start+gindex, 0)

    def catalog_data(self, dindex):
        """ returns an array of data at column **dindex** for all glasses """
        return self.xl_data.col_values(dindex, self.data_start,
                                       self.data_start+self.num_glasses)

    def glass_coefs(self, gindex):
        """ returns an array of glass coefficients for the glass at **gindex** """
        return (self.xl_data.row_values(self.data_start+gindex,
                                        self.coef_col_offset,
                                        self.coef_col_offset+6))

    def glass_map_data(self, wvl='d'):
        """ return index and dispersion data for all glasses in the catalog

        Args:
            wvl (str): the central wavelength for the data, either 'd' or 'e'

        Returns:
            index, V-number, partial dispersion, and glass names
        """
        if wvl == 'd':
            return self.get_glass_map_arrays('nd', 'nF', 'nC')
        elif wvl == 'e':
            return self.get_glass_map_arrays('ne', 'nF', 'nC')
        else:
            return None

    def get_glass_map_arrays(self, nd_str, nf_str, nc_str):
        """ return index and dispersion data arrays for input spectral range

        Args:
            nd_str (str): central wavelength string
            nf_str (str): blue end wavelength string
            nc_str (str): red end wavelength string

        Returns:
            index, V-number, partial dispersion, and glass names
        """
        nd = np.array(
                self.catalog_data(self.data_index(self.nline_str[nd_str])))
        nF = np.array(
                self.catalog_data(self.data_index(self.nline_str[nf_str])))
        nC = np.array(
                self.catalog_data(self.data_index(self.nline_str[nc_str])))
        dFC = nF-nC
        vd = (nd - 1.0)/dFC
        PCd = (nd-nC)/dFC
        names = self.catalog_data(self.name_col_offset)
        return nd, vd, PCd, names


class Glass:
    """ base optical glass

    Attributes:
        gname: the glass name
        gindex: the index into the glass list
        catalog: the GlassCatalog this glass is associated with.
                 **Must be provided by the derived class**
    """
    def __init__(self, gname):
        self.gindex = self.catalog.glass_index(gname)
        self.gname = gname

    def __str__(self):
        return self.catalog.name + ' ' + self.name() + ': ' + self.glass_code()

    def __repr__(self):
        return "{!s}('{}')".format(type(self).__name__, self.gname)

    def sync_to_restore(self):
        """ hook routine to restore the gindex given gname """
        self.gindex = self.catalog.glass_index(self.gname)

    def glass_code(self, nd_str, vd_str):
        """ returns the 6 digit glass code, combining index and V-number """
        nd = self.glass_item(nd_str)
        vd = self.glass_item(vd_str)
        return str(1000*round((nd - 1), 3) + round(vd/100, 3))

    def glass_data(self):
        """ returns the raw spreadsheet data for the glass """
        return self.catalog.glass_data(self.gindex)

    def name(self):
        """ returns the glass name, :attr:`gname` """
        return self.gname

    def glass_item(self, dname):
        """ return the value of the **dname** item

        Args:
            dname (str): header string for data

        Returns:
            the **dname** data

        Raises:
            GlassDataNotFoundError: if **dname** doesn't match any header string
        """
        dindex = self.catalog.data_index(dname)
        if dindex is None:
            return None
        else:
            return self.glass_data()[dindex]

    def rindex(self, wvl):
        """ returns the interpolated refractive index at wvl

        Args:
            wvl: either the wavelength in nm or a string with a spectral line
                 identifier. for the refractive index query

        Returns:
            float: the refractive index at wv_nm

        Raises:
            KeyError: if ``wvl`` is not in the spectra dictionary
        """
        if isinstance(wvl, float):
            return self.calc_rindex(wvl)
        elif isinstance(wvl, int):
            return self.calc_rindex(wvl)
        else:
            return self.calc_rindex(spectra[wvl])

    def calc_rindex(self, wv_nm):
        """ returns the interpolated refractive index at wv_nm

        **Must be provided by the derived class**

        Args:
            wv_nm (float): wavelength in nm for the refractive index query

        Returns:
            float: the refractive index at wv_nm
        """
        pass



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed Jan  3 12:50:03 2018

@author: Mike
"""

import sys

from PyQt5.QtWidgets import (QApplication, QMainWindow, QWidget, QHBoxLayout,
                             QVBoxLayout, QSizePolicy, QGroupBox, QCheckBox,
                             QTableWidget, QTableWidgetItem, QRadioButton)
from PyQt5.QtCore import Qt as qt

from matplotlib.backends.backend_qt5agg \
     import (FigureCanvasQTAgg as FigureCanvas,
             NavigationToolbar2QT as NavigationToolbar)

from matplotlib.figure import Figure

import opticalglass.glassfactory as gf

pickTableHeader = ["Catalog", "Glass", "Nd", "Vd", "P C,d"]
pickTableFormat = ["s", "s", "7.5f", "5.2f", "6.4f"]


class GlassMapViewer(QMainWindow):
    def __init__(self):
        super().__init__()
        self.title = 'Glass Map Viewer'
        self.left = 50
        self.top = 150
        self.width = 1100
        self.height = 650
        self.dataSets = gf.GlassMapModel()
        self.displayDataSets = [True, True, True, True]
        self.display_ref_index = True
        self.initUI()

    def initUI(self):
        self._main = QWidget()
        self.setCentralWidget(self._main)
        layout = QHBoxLayout(self._main)

        self.setWindowTitle(self.title)
        self.setGeometry(self.left, self.top, self.width, self.height)

        self.gmt = self.createTable()
        self.gm = PlotCanvas(self, width=5, height=4)
        self.addToolBar(qt.BottomToolBarArea,
                        NavigationToolbar(self.gm, self))
        layout.addWidget(self.gm)

        rightBar = QVBoxLayout()
        layout.addLayout(rightBar)

        plotTypeGroup = self.createPlotTypeBox()
        rightBar.addWidget(plotTypeGroup)

        catalogGroup = self.createCatalogGroupBox()
        rightBar.addWidget(catalogGroup)

        rightBar.addWidget(self.gmt)

    def createPlotTypeBox(self):
        groupBox = QGroupBox("Plot Type", self)

        index_btn = QRadioButton("Refractive Index")
        index_btn.setChecked(True)
        index_btn.toggled.connect(lambda:
                                  self.on_plot_type_toggled(index_btn))
        partial_btn = QRadioButton("Partial Dispersion")
        partial_btn.toggled.connect(lambda:
                                    self.on_plot_type_toggled(partial_btn))

        vbox = QVBoxLayout()
        vbox.addWidget(index_btn)
        vbox.addWidget(partial_btn)

        groupBox.setLayout(vbox)

        return groupBox

    def on_plot_type_toggled(self, button):
        if button.text() == "Refractive Index":
            if button.isChecked() is True:
                self.display_ref_index = True

        if button.text() == "Partial Dispersion":
            if button.isChecked() is True:
                self.display_ref_index = False

        self.gm.display_ref_index = self.display_ref_index
        self.gm.update_data()
        self.gm.plot()

    def createCatalogGroupBox(self):
        groupBox = QGroupBox("Glass Catalogs", self)

        cdgm_checkBox = QCheckBox("&CDGM")
        cdgm_checkBox.setChecked(True)
        cdgm_checkBox.stateChanged.connect(self.cdgm_check)
        hoya_checkBox = QCheckBox("&Hoya")
        hoya_checkBox.setChecked(True)
        hoya_checkBox.stateChanged.connect(self.hoya_check)
        ohara_checkBox = QCheckBox("&Ohara")
        ohara_checkBox.setChecked(True)
        ohara_checkBox.stateChanged.connect(self.ohara_check)
        schott_checkBox = QCheckBox("&Schott")
        schott_checkBox.setChecked(True)
        schott_checkBox.stateChanged.connect(self.schott_check)

        vbox = QVBoxLayout()
        vbox.addWidget(cdgm_checkBox)
        vbox.addWidget(hoya_checkBox)
        vbox.addWidget(ohara_checkBox)
        vbox.addWidget(schott_checkBox)

        groupBox.setLayout(vbox)

        return groupBox

    def createTable(self):
        table = PickTable()
        return table

    def cdgm_check(self, state):
        checked = state == qt.Checked
        self.gm.displayDataSets[gf.CDGM] = checked
        self.gm.updateVisibility(gf.CDGM, checked)

    def hoya_check(self, state):
        checked = state == qt.Checked
        self.gm.displayDataSets[gf.Hoya] = checked
        self.gm.updateVisibility(gf.Hoya, checked)

    def ohara_check(self, state):
        checked = state == qt.Checked
        self.gm.displayDataSets[gf.Ohara] = checked
        self.gm.updateVisibility(gf.Ohara, checked)

    def schott_check(self, state):
        checked = state == qt.Checked
        self.gm.displayDataSets[gf.Schott] = checked
        self.gm.updateVisibility(gf.Schott, checked)


class PickTable(QTableWidget):
    def __init__(self):
        super().__init__(16, len(pickTableHeader))
        self.rowFill = 16
        self.setHorizontalHeaderLabels(pickTableHeader)
        for i, w in enumerate([52, 96, 61, 48, 60]):
            self.setColumnWidth(i, w)
        self.setAlternatingRowColors(True)
        self.setMinimumWidth(285)
        self.setMaximumWidth(335)

    def setRowCount(self, count):
        if count > self.rowFill:
            super().setRowCount(count)

    def resizeEvent(self, event):
        super(QTableWidget, self).resizeEvent(event)
        rowsToFill = int(self.height()/self.rowHeight(0))
        if rowsToFill > self.rowFill:
            self.rowFill = rowsToFill
            if rowsToFill > self.rowCount():
                super().setRowCount(rowsToFill)


class PlotCanvas(FigureCanvas):
    dsc = [(56/255, 142/255, 142/255),  # sgi teal
           (113/255, 113/255, 198/255),  # sgi slateblue
           (102/255, 205/255, 0),  # chartreuse 3
           (255/255, 114/255, 86/255)]  # coral 1

    def __init__(self, parent=None, width=5, height=4, dpi=100):
        # fig = Figure(figsize=(width, height), dpi=dpi)
        fig = Figure()
        self.axes = fig.add_subplot(111)
        self.rawData = []
        self.data = parent.dataSets
        self.displayDataSets = parent.displayDataSets
        self.pickTable = parent.gmt
        self.display_ref_index = parent.display_ref_index
        self.needsClear = True
        self.pickRow = 0

        self.update_data()

        super().__init__(fig)
        self.setParent(parent)

        FigureCanvas.setSizePolicy(self,
                                   QSizePolicy.Expanding,
                                   QSizePolicy.Expanding)

        FigureCanvas.updateGeometry(self)
        self.plot()

    def get_display_label(self):
        if self.display_ref_index is True:
            return 'Glass Map'
        else:
            return 'Partial Dispersion'

    def update_data(self):
        self.rawData = []
        for i, display in enumerate(self.displayDataSets):
            n, v, p, lbl = self.data.get_data_at(i)
            dsLabel = self.data.get_data_set_label_at(i)
            self.rawData.append([dsLabel, (n, v, p, lbl)])

    def plot(self):
        self.axes.cla()
        xi = 1
        if self.display_ref_index is True:
            yi = 0
        else:
            yi = 2
        self.axes.set_title(self.get_display_label())
        for i, display in enumerate(self.displayDataSets):
            self.axes.plot(self.rawData[i][1][xi], self.rawData[i][1][yi],
                           linestyle='None', marker='o', markersize=5,
                           color=self.dsc[i], alpha=0.5, gid=i, picker=5,
                           label=self.rawData[i][0], visible=display)

        self.figure.canvas.mpl_connect('pick_event', self.on_pick)
        self.figure.canvas.mpl_connect('button_press_event', self.on_press)
        self.axes.invert_xaxis()
        self.axes.grid()
        self.axes.legend()
        self.draw()

    def clearPickTable(self):
        self.pickTable.clearContents()
        self.pickRow = 0
        self.needsClear = False

    def on_press(self, event):
        if self.needsClear:
            # If needsClear is still set, there have been no pick events so
            #  this is a click in an empty region of the plot.
            #  Clear the pick table
            self.clearPickTable()
        else:
            # on_press event happens after on_pick events. Set needsClear for
            #  next on_pick, i.e. a new selection, to handle
            self.needsClear = True

    def on_pick(self, event):
        if self.needsClear:
            self.clearPickTable()
        line = event.artist
        id = line.get_gid()
        if self.displayDataSets[id]:
            ind = event.ind
            dsLabel = self.rawData[id][0]
            n, v, p, lbl = self.rawData[id][1]
            self.pickTable.setRowCount(self.pickRow+len(ind))
            for k in ind:
                glass = (dsLabel, lbl[k], n[k], v[k], p[k])
                for j in range(len(pickTableHeader)):
                    item = QTableWidgetItem(format(glass[j],
                                                   pickTableFormat[j]))
                    self.pickTable.setItem(self.pickRow, j, item)
                self.pickRow += 1

    def updateVisibility(self, indx, state):
        self.axes.lines[indx].set_visible(state)
        self.draw()


def main():
    qtapp = QApplication(sys.argv)
    qtwnd = GlassMapViewer()
    qtwnd.show()
    return qtapp.exec_()


if __name__ == '__main__':
    sys.exit(main())



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
""" unit test for Ohara optical glass catalog

.. codeauthor: Michael J. Hayford
"""

import unittest
import opticalglass.ohara as o


class OharaTestCase(unittest.TestCase):
    catalog = o.OharaCatalog()

    def compare_indices(self, glass, tol=5e-6):
        nC = glass.rindex(656.27)
        nd = glass.rindex(587.56)
        ne = glass.rindex(546.07)
        nF = glass.rindex(486.13)
        ng = glass.rindex(435.84)
        nh = glass.rindex(404.66)
        nI = glass.rindex(365.01)
        indxC = glass.glass_data()[self.catalog.data_index('nC')]
        indxd = glass.glass_data()[self.catalog.data_index('nd')]
        indxe = glass.glass_data()[self.catalog.data_index('ne')]
        indxF = glass.glass_data()[self.catalog.data_index('nF')]
        indxg = glass.glass_data()[self.catalog.data_index('ng')]
        indxh = glass.glass_data()[self.catalog.data_index('nh')]
        indxI = glass.glass_data()[self.catalog.data_index('ni')]
        self.assertAlmostEqual(nC, indxC, delta=tol)
        self.assertAlmostEqual(nd, indxd, delta=tol)
        self.assertAlmostEqual(ne, indxe, delta=tol)
        self.assertAlmostEqual(nF, indxF, delta=tol)
        self.assertAlmostEqual(ng, indxg, delta=tol)
        self.assertAlmostEqual(nh, indxh, delta=tol)
        self.assertAlmostEqual(nI, indxI, delta=tol)

    def test_ohara_catalog_glass_index(self):
        sfpl51 = self.catalog.glass_index('S-FPL51')  # first in list
        self.assertEqual(sfpl51, 0)
        stim2 = self.catalog.glass_index('S-TIM 2')
        self.assertEqual(stim2, 41)
        sbsl7 = self.catalog.glass_index('S-BSL 7')
        self.assertEqual(sbsl7, 6)
        snph1 = self.catalog.glass_index('S-NPH 1')
        self.assertEqual(snph1, 126)
        snph53 = self.catalog.glass_index('S-NPH53')  # last in list
        self.assertEqual(snph53, 133)

    def test_ohara_catalog_data_index(self):
        nd = self.catalog.data_index('nd')
        self.assertEqual(nd, 16)
        vd = self.catalog.data_index('νd')
        self.assertEqual(vd, 24)
        B1 = self.catalog.data_index('B1')
        self.assertEqual(B1, 63)
        glasscode = self.catalog.data_index('Code(d)')
        self.assertEqual(glasscode, 2)
        date = self.catalog.data_index(' D0')
        self.assertEqual(date, 154)

    def test_ohara_glass_stim2(self):
        glass = o.OharaGlass('S-TIM 2')
        self.assertIsNotNone(glass.gindex)
        self.assertEqual(glass.name(), 'S-TIM 2')
        self.compare_indices(glass, tol=6e-6)

    def test_ohara_glass_sbsl7(self):
        glass = o.OharaGlass('S-BSL 7')
        self.assertIsNotNone(glass.gindex)
        self.assertEqual(glass.name(), 'S-BSL 7')
        self.compare_indices(glass, tol=6e-6)

    def test_ohara_glass_snbh53v(self):
        glass = o.OharaGlass('S-NBH53V')
        self.assertIsNotNone(glass.gindex)
        self.assertEqual(glass.name(), 'S-NBH53V')
        self.compare_indices(glass)


if __name__ == '__main__':
    unittest.main(verbosity=2)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Copyright © 2018 Michael J. Hayford
""" Interfaces for commercial glass catalogs

.. codeauthor: Michael J. Hayford
"""
import logging

from . import cdgm as c
from . import hoya as h
from . import ohara as o
from . import schott as s

from . import glasserror as ge

CDGM, Hoya, Ohara, Schott = range(4)
_cat_names = ["CDGM", "Hoya", "Ohara", "Schott"]


def create_glass(name, catalog):
    """ Factory function returning a catalog glass instance.

    Arguments:
        name: glass name
        catalog: name of supported catalog (CDGM, Hoya, Ohara, Schott)

    Raises:
        GlassCatalogNotFoundError: if catalog isn't found
        GlassNotFoundError: if name isn't in the specified catalog

    """
    cat_name = catalog.capitalize()
    glass_name = name.upper()
    if catalog.upper() == _cat_names[CDGM]:
        return c.CDGMGlass(glass_name)
    elif cat_name == _cat_names[Hoya]:
        return h.HoyaGlass(glass_name)
    elif cat_name == _cat_names[Ohara]:
        return o.OharaGlass(glass_name)
    elif cat_name == _cat_names[Schott]:
        return s.SchottGlass(glass_name)
    else:
        logging.info('Glass catalog %s not found', catalog)
        raise ge.GlassCatalogNotFoundError(catalog)
        return None


def get_glass_catalog(catalog):
    """ Function returning a glass catalog instance.

    Arguments:
        catalog: name of supported catalog (CDGM, Hoya, Ohara, Schott)

    Raises:
        GlassCatalogNotFoundError: if catalog isn't found
    """
    cat_name = catalog.capitalize()
    if catalog.upper() == _cat_names[CDGM]:
        return c.CDGMGlass
    elif cat_name == _cat_names[Hoya]:
        return h.HoyaGlass
    elif cat_name == _cat_names[Ohara]:
        return o.OharaGlass
    elif cat_name == _cat_names[Schott]:
        return s.SchottGlass
    else:
        logging.info('Glass catalog %s not found', catalog)
        raise ge.GlassCatalogNotFoundError(catalog)
        return None


class GlassMapModel():
    """ Simple model to support Model/View architecture for Glass map views """
    def __init__(self):
        self.dataSetList = []
        self.dataSetList.append((c.CDGMCatalog(), _cat_names[CDGM]))
        self.dataSetList.append((h.HoyaCatalog(), _cat_names[Hoya]))
        self.dataSetList.append((o.OharaCatalog(), _cat_names[Ohara]))
        self.dataSetList.append((s.SchottCatalog(), _cat_names[Schott]))

    def get_data_at(self, i):
        return self.dataSetList[i][0].glass_map_data()

    def get_data_set_label_at(self, i):
        return self.dataSetList[i][1]



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Copyright © 2017 Michael J. Hayford
""" Support for the Hoya Glass catalog

.. codeauthor: Michael J. Hayford
"""
from math import sqrt

from . import glass


class HoyaCatalog(glass.GlassCatalog):
    #    data_header = 1
    #    data_start = 4
    #    num_glasses = 194
    #    name_col_offset = 2
    #    coef_col_offset = 28
    #    index_col_offset = 10
    nline_str = {'nC': 'nC',
                 'nd': 'nd',
                 'ne': 'ne',
                 'nF': 'nF',
                 'ng': 'ng',
                 'nh': 'nh',
                 'ni': 'ni'}

    def __init__(self, fname='HOYA.xlsx'):
        super().__init__('Hoya', fname, 'Glass\u3000Type', 'A0', 'n1529.6',
                         data_header_offset=1)

    def glass_coefs(self, gindex):
        c = (self.xl_data.row_values(self.data_start+gindex,
                                     self.coef_col_offset,
                                     self.coef_col_offset+12))
        return [x*10**y for x, y in zip(c[::2], c[1::2])]


class HoyaGlass(glass.Glass):
    catalog = HoyaCatalog()

    def __init__(self, gname, catalog=None):
        if catalog is not None:
            self.catalog = catalog
        super().__init__(gname)

    def glass_code(self):
        return super().glass_code('nd', 'νd')

    def calc_rindex(self, wv_nm):
        wv = 0.001*wv_nm
        wv2 = wv*wv
        coefs = self.catalog.glass_coefs(self.gindex)
        n2 = coefs[0] + coefs[1]*wv2
        wvm2 = 1/wv2
        n2 = n2 + wvm2*(coefs[2] + wvm2*(coefs[3]
                        + wvm2*(coefs[4] + wvm2*coefs[5])))
        return sqrt(n2)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Copyright © 2017 Michael J. Hayford
""" Support for the CDGM Glass catalog

.. codeauthor: Michael J. Hayford
"""
import logging

from math import sqrt

from . import glass


class CDGMCatalog(glass.GlassCatalog):
    #    data_header = 0
    #    data_start = 2
    #    num_glasses = 240
    #    name_col_offset = 0
    #    coef_col_offset = 21
    #    index_col_offset = 2
    nline_str = {'nC': 'nc',
                 'nd': 'nd',
                 'ne': 'ne',
                 'nF': 'nF',
                 'ng': 'ng',
                 'nh': 'nh',
                 'ni': 'ni'}

    def __init__(self, fname='CDGM.xls'):
        super().__init__('CDGM', fname, 'Glass', 'A0', 'nt')

    def get_glass_names(self):
        """ returns a list of glass names """
        gnames = self.xl_data.col_values(self.name_col_offset, self.data_start)
        # filter out any empty cells at the end
        while gnames and (len(gnames[-1]) is 0 or gnames[-1] == 'Over!'):
            gnames.pop()
        return gnames


class CDGMGlass(glass.Glass):
    catalog = CDGMCatalog()

    def __init__(self, gname, catalog=None):
        if catalog is not None:
            self.catalog = catalog
        super().__init__(gname)

    def glass_code(self):
        return super().glass_code('nd', 'υd')

    def calc_rindex(self, wv_nm):
        wv = 0.001*wv_nm
        wv2 = wv*wv
        coefs = self.catalog.glass_coefs(self.gindex)
        n2 = coefs[0] + coefs[1]*wv2
        wvm2 = 1/wv2
        n2 = n2 + wvm2*(coefs[2] + wvm2*(coefs[3]
                        + wvm2*(coefs[4] + wvm2*coefs[5])))
        return sqrt(n2)



""" package supplying optical glass data from vendor supplied data

    The ``opticalglass`` package currently supports the following vendors:

        - CDGM
        - Hoya
        - Ohara
        - Schott

    The spreadsheets from the vendors are in the ``data`` directory.
"""
